@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2005 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.44.2.4 2005/01/30 19:40:23 vapier Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.44.2.10 2005/05/17 00:40:41 vapier Exp $
 
 # Common functions
 [[ ${RC_GOT_FUNCTIONS} != "yes" ]] && source /sbin/functions.sh
@@ -10,8 +10,8 @@
 # Functions to control daemons
 [[ ${RC_GOT_DAEMON} != "yes" ]] && source "${svclib}/sh/rc-daemon.sh"
 
-# Fix bug 48595
-if [[ ${EUID} != 0 ]] ; then
+# User must be root to run most script stuff (except status)
+if [[ ${EUID} != 0 ]] && ! [[ $2 == "status" && $# -eq 2 ]] ; then
 	eerror "$0: must be root to run init scripts"
 	exit 1
 fi
@@ -28,6 +28,7 @@ else
 fi
 
 myservice=${myservice##*/}
+export SVCNAME=${myservice}
 mylevel=$(<"${svcdir}/softlevel")
 
 
@@ -39,7 +40,16 @@ else
 	IFACE=
 	NETSERVICE=
 fi
-		
+
+# We need to stop hotplug from launching net scripts until the boot level
+# has completed
+if [[ ${NETSERVICE} == "yes" && ${IFACE} != "lo" ]] ; then
+	if [[ -z ${mylevel} || ${mylevel} == ${BOOTLEVEL} ]] ; then
+		eerror "$0: cannot start until the runlevel "${BOOTLEVEL}" has completed"
+		exit 1
+	fi
+fi
+
 # Source configuration files.
 # (1) Source /etc/conf.d/${myservice} to get initscript-specific
 #     configuration (if it exists).
@@ -278,7 +288,15 @@ svc_start() {
 			eerror "        services.  \"${myservice}\" was not started."
 			retval=1
 		elif [[ ${retval} -eq 0 ]] && ! broken "${myservice}" ; then
+			(
+			exit() {
+				eerror "DO NOT USE EXIT IN INIT.D SCRIPTS"
+				eerror "This IS a bug, please fix your broken init.d"
+				unset -f exit
+				exit $@
+			}
 			start
+			)
 			retval="$?"
 		fi
 
@@ -318,25 +336,23 @@ svc_status() {
 	# as an generic means to detect status.  Any other output
 	# should thus be formatted in the custom status() function
 	# to work with the printed " * status:  foo".
+	local efunc="" state=""
 
 	if service_started "${myservice}" ; then
-		if [[ ${RC_QUIET_STDOUT} != "yes" ]] ; then
-			einfo "status:  started"
-		else
-			return 0
-		fi
+		efunc="einfo"
+		state="started"
 	else
-		if [[ ${RC_QUIET_STDOUT} != "yes" ]] ; then
-			eerror "status:  stopped"
-		else
-			return 1
-		fi
+		efunc="eerror"
+		state="stopped"
 	fi
+	[[ ${RC_QUIET_STDOUT} != "yes" ]] \
+		&& ${efunc} "status:  ${state}"
 
 	status
+	[[ ${efunc} != "eerror" ]]
 }
 
-rcscript_errors=$(wrap_rcscript "${myscript}" 2>&1) || {
+rcscript_errors=$(bash -n "${myscript}" 2>&1) || {
 	[[ -n ${rcscript_errors} ]] && echo "${rcscript_errors}" >&2
 	eerror "ERROR:  \"${myscript}\" has syntax errors in it; not executing..."
 	exit 1
